/*
 * MaFenetre.cpp
 * This file is part of ChronoCheckMate
 *
 * Copyright (C) 2010 - François Drouhard
 *
 * ChronoCheckMate is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ChronoCheckMate is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ChronoCheckMate; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */
 
#include "MaFenetre.h"
#include "Chronometre.h"
#include <QMenuBar>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QPushButton>
#include <QKeyEvent>
#include <QApplication>
#include <QLabel>
#include <QDialog>
#include <QLineEdit>
#include <QDialogButtonBox>

MaFenetre::MaFenetre() :
    QMainWindow (),
    marche(false),
    nomJoueur1 ("Joueur 1"),
    nomJoueur2 ("Joueur 2")
{
    QWidget *centralWidget = new QWidget;

    chrono1 = new Chronometre(nomJoueur1 , this);
    chrono2 = new Chronometre(nomJoueur2 , this);
    boutonInverser = new QPushButton ("<->");
    boutonInverser->setMinimumWidth(50);
    boutonInverser->setMaximumWidth(50);

	////////////////// Groupe Définition de la classe ///////////////////////////
    layoutChrono = new QHBoxLayout;
    layoutChrono->addWidget(chrono1);
    layoutChrono->addWidget(boutonInverser, Qt::AlignVCenter);
    layoutChrono->addWidget(chrono2);

    ///////////////////////////// Menus ////////////////////////////
    QMenu *menuFichier = menuBar()->addMenu("&Fichier");
    QAction *actionNouveau = new QAction ("&Nouveau" , this);
    QAction *actionChangerNom = new QAction ("&Changer nom des joueurs...", this);
    QAction *actionQuitter = new QAction ("&Quitter" , this);
    menuFichier->addAction(actionNouveau);
    menuFichier->addAction(actionChangerNom);
    menuFichier->addSeparator();
    menuFichier->addAction(actionQuitter);

	///////////////////////////// Boutons   /////////////////////////////
    boutonDemarrer = new QPushButton ("Commencer (Barre Espace)");
    boutonArreter = new QPushButton ("Arrêter(Touche Entrée)");
	
	boutonArreter->setFocusPolicy (Qt::NoFocus);
    boutonInverser->setFocusPolicy(Qt::NoFocus);

    etatBoutons (true , false);
	
	layoutBoutonSwitch = new QHBoxLayout;
	layoutBoutonSwitch->addWidget (boutonDemarrer);
    layoutBoutonSwitch->addWidget (boutonArreter);

    ///////////////////////////// Aide /////////////////////////////////////
    QLabel *labelAide = new QLabel ("<b>Aide</b><br />Espace : Lancer le chrono/Donner le chrono<br />Entrée : Stopper les deux chronos");
	
	/////////////////////////////Mise en page finale ///////////////////////////
	layoutComplet = new QVBoxLayout;
	layoutComplet->addLayout (layoutChrono);
	layoutComplet->addLayout (layoutBoutonSwitch);
    layoutComplet->addWidget (labelAide);
    centralWidget->setLayout (layoutComplet);
    setCentralWidget(centralWidget);
	setWindowTitle ("ChronoCheckMate");

    ///////////////////////////// Ouverture de la boite de dialogue /////////////////
    definirNomJoueurs();

	///////////////////////////// connections ///////////////////////////
    QObject::connect (boutonDemarrer , SIGNAL(clicked () ) , this , SLOT(demarrer () ));
    QObject::connect (boutonInverser , SIGNAL(clicked()) , this , SLOT(inverser()));
    QObject::connect (boutonArreter , SIGNAL(clicked()), this, SLOT (arreter()));
    QObject::connect (actionNouveau, SIGNAL(triggered()) , this , SLOT(init()));
    QObject::connect (actionChangerNom , SIGNAL(triggered()), this, SLOT(definirNomJoueurs()));
    QObject::connect (actionQuitter , SIGNAL(triggered()) , qApp , SLOT(quit()));
}

///////////////// Méthodes ////////////////////
void MaFenetre::etatBoutons (bool etatBoutonStart, bool etatBoutonStop) {
    boutonDemarrer->setEnabled (etatBoutonStart);
    boutonArreter->setEnabled (etatBoutonStop);
}

////////////////// SLOTS ////////////////////////
void MaFenetre::inverser() {
    layoutChrono->removeWidget(chrono1);
    layoutChrono->removeWidget(chrono2);
    Chronometre *chronoTemp = chrono1;
    chrono1 = chrono2;
    chrono2 = chronoTemp;
    layoutChrono->insertWidget(0 , chrono1);
    layoutChrono->addWidget(chrono2);
}

void MaFenetre::demarrer () {
	if (marche == false ) {
		chrono1->reinit ();
		chrono2->reinit ();
		chrono1->start ();
		marche = true;
        etatBoutons (false , true);
	}
}

void MaFenetre::modifierNomJoueurs() {
    chrono1->modifierNomJoueur(nomJoueur1);
    chrono2->modifierNomJoueur(nomJoueur2);

}

void MaFenetre::switcher () {
	if (marche == true) {
		chrono1->basculer ();
		chrono2->basculer ();
	}
}

void MaFenetre::arreter () {
	chrono1->stop();
	chrono2->stop();
	marche = false;
    etatBoutons (true , false);
}

void MaFenetre::init () {
    chrono1->reinit();
    chrono2->reinit();
    marche = false;
    etatBoutons (true , false);
}

void MaFenetre::definirNomJoueurs() {
    QDialog *dialog = new QDialog (this);
    QHBoxLayout *layoutLineEdit = new QHBoxLayout;
    QVBoxLayout *layoutDialog = new QVBoxLayout;
    QDialogButtonBox *boutonJouer = new QDialogButtonBox (QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
    QLineEdit *joueur1LineEdit = new QLineEdit(nomJoueur1 , this);
    QLineEdit *joueur2LineEdit = new QLineEdit(nomJoueur2 , this);
    joueur1LineEdit->selectAll();

    layoutLineEdit->addWidget(joueur1LineEdit);
    layoutLineEdit->addWidget(joueur2LineEdit);
    layoutDialog->addLayout(layoutLineEdit);
    layoutDialog->addWidget(boutonJouer);
    dialog->setLayout(layoutDialog);

    QObject::connect(boutonJouer , SIGNAL(accepted()) , dialog, SLOT(accept()));
    QObject::connect(boutonJouer , SIGNAL(rejected()) , dialog, SLOT(close()));

    if (dialog->exec()) {
        nomJoueur1 = joueur1LineEdit->text();
        nomJoueur2 = joueur2LineEdit->text();
        modifierNomJoueurs();
    }
}

void MaFenetre::keyReleaseEvent (QKeyEvent * evenement) {
	if (evenement->key() == Qt::Key_Space) {
        if (marche == true) {
            switcher();
        }
        if (marche == false) {
            demarrer();
        }
	}
    if (evenement->key() == Qt::Key_Return) {
        arreter();
    }
}
